import datetime

from boox.models.config_users import SyncTokenResponse


def test_sync_token_response_parses_token_expiration_date_correctly():
    token_expiry = datetime.datetime.now(tz=datetime.UTC).replace(microsecond=0) + datetime.timedelta(days=180)
    data = SyncTokenResponse.model_validate({
        "data": None,
        "message": "SUCCESS",
        "result_code": 0,
        "tokenExpiredAt": int(token_expiry.timestamp()),
    })
    assert not data.data
    assert data.token_expired_at == token_expiry



import datetime
import re

import pytest
from pydantic import ValidationError

from boox.models.users import (
    DataSession,
    DataToken,
    FetchTokenRequest,
    SendVerifyCodeRequest,
    SyncSessionTokenResponse,
    soft_validate_email,
)

EMAIL = "foo@bar.com"
INVALID_EMAIL = "foobar@baz"


def test_soft_validate_email_returns_true_for_valid_email():
    is_email = soft_validate_email(EMAIL)
    assert is_email


def test_soft_validate_email_returns_false_for_invalid_email():
    is_email = soft_validate_email(INVALID_EMAIL)
    assert not is_email


def test_validation_fails_when_mobi_is_empty_string():
    with pytest.raises(ValidationError, match="String should have at least 6 characters"):
        SendVerifyCodeRequest.model_validate({"mobi": ""})


def test_validation_requires_area_code_for_phone_number():
    with pytest.raises(ValidationError, match="area_code must be provided if phone method is used"):
        SendVerifyCodeRequest.model_validate({"mobi": "123456789"})


def test_validation_fails_when_area_code_does_not_match_pattern():
    with pytest.raises(ValidationError, match="String should match pattern"):
        SendVerifyCodeRequest.model_validate({"mobi": "123456789", "area_code": "0048"})


def test_validation_fails_when_mobi_is_neither_email_nor_phone():
    with pytest.raises(ValidationError, match="mobi field must either be an e-mail or a phone number"):
        SendVerifyCodeRequest.model_validate({"mobi": INVALID_EMAIL})


def test_validation_fails_when_email_and_area_code_are_both_provided():
    with pytest.raises(ValidationError, match="mobi and area_code are mutually exclusive"):
        SendVerifyCodeRequest.model_validate({"mobi": EMAIL, "area_code": "+48"})


def test_validation_allows_email_without_area_code():
    assert SendVerifyCodeRequest.model_validate({"mobi": EMAIL})


def test_validation_fails_when_verification_code_does_not_match_pattern():
    with pytest.raises(ValidationError, match="String should match pattern"):
        FetchTokenRequest.model_validate({"mobi": EMAIL, "code": "1234567"})


def test_validation_succeeds_when_verification_code_matches_pattern():
    assert FetchTokenRequest.model_validate({"mobi": EMAIL, "code": "123456"})


def test_token_str_is_masked():
    data = DataToken.model_validate({"token": "xyz123"})
    assert re.compile(r"\*+").fullmatch(str(data.token))


def test_token_is_public_in_model_dump():
    value = "xyz123"
    data = DataToken.model_validate({"token": value})
    dumped = data.model_dump()
    assert dumped.get("token") == value


def test_token_is_public_in_json_dump():
    value = "xyz123"
    data = DataToken.model_validate({"token": value})
    dumped = data.model_dump_json()
    assert re.compile(r'{"token":"xyz123"}').fullmatch(dumped)


def test_sync_session_token_response_parses_nested_data_correctly():
    session_expiry = datetime.datetime.now(tz=datetime.UTC).replace(microsecond=0)
    token_expiry = datetime.datetime.now(tz=datetime.UTC).replace(microsecond=0) + datetime.timedelta(days=180)
    data = SyncSessionTokenResponse.model_validate({
        "data": {
            "channels": (),
            "cookie_name": "foo",
            "expires": session_expiry.strftime("%Y-%m-%dT%H:%M:%SZ"),
            "session_id": "bar",
        },
        "message": "SUCCESS",
        "result_code": 0,
        "tokenExpiredAt": int(token_expiry.timestamp()),
    })
    assert isinstance(data.data, DataSession)
    assert data.token_expired_at == token_expiry
    assert data.data.expires == session_expiry



"""Example on how to synchronize session token.

While this method is not required at all, especially if you write short scripts,
it makes sense to use it occasionally to make sure that you don't use an expired token.
But you might want to use it only if you rely on the session_id.
"""

import datetime
from typing import cast

from pydantic import SecretStr

from boox.core import Boox
from boox.models.enums import BooxUrl
from boox.models.users import SyncSessionTokenResponse

TOKEN = cast(str, ...)  # Just to make this example easier


with Boox(base_url=BooxUrl.EUR, token=TOKEN) as boox:
    response: SyncSessionTokenResponse = boox.users.synchronize_session_token()

_session_id: SecretStr = response.data.session_id
_expires: datetime.datetime = response.data.expires



import pytest
from pydantic import ValidationError

from boox.models.base import BaseResponse, BaseSyncResponse


def test_base_response_str_format():
    response = BaseResponse[None](data=None, message="foo", result_code=123)
    assert str(response) == "<123: foo>"


def test_subclass_response_repr_format():
    class DummyResponse(BaseResponse[None]): ...

    response = DummyResponse(data=None, message="foo", result_code=123)
    assert repr(response) == "DummyResponse(<123: foo>)"


def test_base_sync_subclass_raises_error_if_no_token_expiry_date_provided():
    class DummySyncResponse(BaseSyncResponse[None]): ...

    with pytest.raises(ValidationError, match=r"Field required"):
        DummySyncResponse.model_validate({"data": None, "message": "foo", "result_code": 123})



from boox.api.core import Api, requires_token
from boox.models.config_users import SyncTokenResponse


class ConfigUsersApi(Api):
    """API wrappers for `configUsers` endpoint family.

    Note that since Boox class already has ConfigUsersApi in its context,
    it is not recommended to use ConfigUsersApi as a standalone object.
    """

    @requires_token
    def synchronize_token(self) -> SyncTokenResponse:
        """A call to check token authenticity and validity.

        A typical scenario for this route is to use it before any action to prevent a server error.

        This call **requires** the token to be passed as an Authorization header, e.g.:
            >>> {"Authorization": "Bearer xyz123abc"}
        That is also the reason why this call pre-validates the client header.

        Returns:
            SyncTokenResponse: The validated response containing information about token expiry date.
        """
        response = self._get(endpoint="/api/1/configUsers/one")
        return SyncTokenResponse.model_validate(response.json())



from collections.abc import Iterator
from contextlib import suppress

import pytest

from tests.api.utils import E2EConfig, EmailProvider


@pytest.fixture(scope="session")
def config() -> E2EConfig:
    return E2EConfig()


@pytest.fixture(scope="session")
def email(config: E2EConfig) -> Iterator[EmailProvider]:
    """An email provider for connecting to an SMTP server.

    Useful for getting the verification code.
    At the end of the session all messages in the inbox are cleaned-up.

    Yields:
        EmailProvider: a testing-only wrapper on httpx.Client.
    """
    provider = EmailProvider(config)
    yield provider
    with suppress(ValueError):
        provider.cleanup_inbox()



"""Example on how to send the verification code.

This basically, in many situations, is your first step when using the library.

The example is divided into 2 parts:
    - robust, using all available types,
    - simplified, which still should pass (in case you have very loose static analysis rules)

Bear in mind that since this library is heavily validated on many steps,
You probably are ok with the simplified version.
Mix and match however you like.
"""

# Example 1: robust
from boox import Boox, BooxUrl
from boox.models.users import SendVerifyCodeRequest, SendVerifyResponse

with Boox(base_url=BooxUrl.EUR) as boox:
    payload = SendVerifyCodeRequest.model_validate({"mobi": "foo@bar.com"})
    _: SendVerifyResponse = boox.users.send_verification_code(payload=payload)

# Example 2: simplified
# pyright: reportArgumentType=false

# Here, in this example I don't use a context manager, and I close the connection manually.
# It does not mean that you have to use it like that.
# In fact, I highly recommend using Boox as a context manager.
# This is similar to using builtins.open().

boox = Boox(base_url="eur.boox.com")
payload = {"mobi": "foo@bar.com"}
_: SendVerifyResponse = boox.users.send_verification_code(payload=payload)
boox.close()



import re
import warnings
from typing import cast

from pydantic import ConfigDict, SecretStr, TypeAdapter, validate_call

from boox.api.config_users import ConfigUsersApi
from boox.api.users import UsersApi
from boox.client import BaseHttpClient
from boox.models.enums import BooxUrl
from boox.models.protocols import HttpClient


class Boox:
    """The client used to communicate with the BOOXDrop remote server.

    It is not meant to be used with the local connection (via USB protocol) as it has a completely different API.

    Examples:
        Example 1, using as a context manager.

        >>> # Given it is the very first connection, and no token is available:
        >>> with Boox(base_url="https://eur.boox.com") as client:
        ...     payload = {"mobi": "foo@bar.com"}
        ...     client.users.send_verification_code(payload=payload)
        SendVerifyResponse(<0: SUCCESS>)

        Example 2, closing the connection manually.
        Notice that you can also use BooxUrl enum to not rely on strings for the base_url

        >>> from boox.models.enums import BooxUrl
        >>> client = Boox(base_url=BooxUrl.EUR)
        >>> payload = {"mobi": "foo@bar.com"}
        >>> client.users.send_verification_code(payload=payload)
        SendVerifyResponse(<0: SUCCESS>)
        >>> client.close()
    """

    def __init__(
        self, client: HttpClient | None = None, base_url: BooxUrl | None = None, token: str | SecretStr = ""
    ) -> None:
        if is_closed := getattr(client, "is_closed", False):
            raise ValueError("Cannot initialize Boox with a closed connection")

        if base_url := (getattr(client, "base_url", None) or base_url):
            base_url = TypeAdapter(BooxUrl).validate_python(base_url)

        self._is_closed: bool = is_closed
        self._base_url: str | None = base_url
        self.client: HttpClient = client or BaseHttpClient()
        self.client.headers.update({"Content-Type": "application/json"})
        if not self.client.headers.get("Authorization") and token:
            if isinstance(token, SecretStr):
                token = token.get_secret_value()
            self.client.headers.update({"Authorization": f"Bearer {token}"})

        self.users = UsersApi(self)
        self.config_users = ConfigUsersApi(self)

    def __enter__(self):
        return self

    def __exit__(self, *_):
        self.close()

    def __del__(self):
        if getattr(self, "is_closed", True) is False:
            warnings.warn(
                "Boox client was not closed explicitly (neither via `with` nor `.close()`)",
                ResourceWarning,
                stacklevel=2,
            )
            self.close()

    @property
    def base_url(self) -> str | None:
        """Property to conveniently store and set the base_url that will be used for API calls.

        Returns:
            str | None: Please refer to `BooxUrl`.
        """
        return self._base_url

    @base_url.setter
    @validate_call(config=ConfigDict(use_enum_values=True))
    def base_url(self, value: BooxUrl):
        self._base_url = value

    @property
    def token(self):
        """Property to conveniently store and set the authorization token for the majority of API calls.

        Returns:
            str: The token itself, plain, not protected.
        """
        header = cast(str, self.client.headers.get("Authorization", ""))
        if match := re.compile(r"Bearer (?P<token>.+)").fullmatch(header):
            return cast(str, match.group("token"))
        return ""

    @token.setter
    def token(self, value: SecretStr | str):
        if isinstance(value, SecretStr):
            value = value.get_secret_value()
        self.client.headers.update({"Authorization": f"Bearer {value}"})

    @property
    def is_closed(self):
        """Property to check whether a client's connection is closed."""
        return self._is_closed

    def close(self):
        """An explicit way of closing the Boox client."""
        if self.is_closed or getattr(self, "client", None) is None:
            return
        self.client.close()
        self._is_closed = True



import gc
import warnings
from collections.abc import Callable
from unittest import mock
from uuid import UUID, uuid1

import pytest
from pydantic import SecretStr, ValidationError
from pytest_mock import MockerFixture

from boox.api.config_users import ConfigUsersApi
from boox.api.users import UsersApi
from boox.core import Boox
from boox.models.enums import BooxUrl

# pyright: reportPrivateUsage=false


def _shows_all_members(e: ValidationError) -> bool:
    errors = e.errors(include_url=False, include_context=False, include_input=False)
    return all(m.value in errors[0]["msg"] for m in list(BooxUrl))


def _cast_to_secret_str(u: UUID) -> SecretStr:
    return SecretStr(str(u))


def test_boox_initializes_with_defaults():
    assert Boox()


def test_boox_base_url_is_none_by_default():
    boox = Boox()
    assert boox.base_url is None


@pytest.mark.parametrize("url", list(BooxUrl))
def test_boox_base_url_inferred_from_client(mocked_client: mock.Mock, url: BooxUrl):
    mocked_client.base_url = url
    boox = Boox(client=mocked_client)
    assert boox.base_url is url


def test_explicit_base_url_takes_precedence(mocked_client: mock.Mock):
    url_taking_precedence = BooxUrl.EUR
    boox = Boox(client=mocked_client, base_url=url_taking_precedence)
    assert boox.base_url is url_taking_precedence


def test_client_base_url_overrides_constructor_value(mocked_client: mock.Mock):
    url_taking_precedence = BooxUrl.EUR
    url_not_taking_precedence = BooxUrl.PUSH
    mocked_client.base_url = url_taking_precedence
    boox = Boox(client=mocked_client, base_url=url_not_taking_precedence)
    assert boox.base_url is url_taking_precedence


def test_boox_raises_validation_error_for_invalid_url(mocked_client: mock.Mock):
    mocked_client.base_url = "http://random.url"
    with pytest.raises(ValidationError, match="Input should be", check=_shows_all_members):
        Boox(client=mocked_client)


@pytest.mark.parametrize("url", list(BooxUrl))
def test_boox_base_url_can_be_set(mocked_client: mock.Mock, url: BooxUrl):
    boox = Boox(client=mocked_client)
    boox.base_url = url
    assert boox.base_url == url.value


def test_boox_base_url_set_raises_on_invalid_url(mocked_client: mock.Mock):
    boox = Boox(client=mocked_client)
    with pytest.raises(ValidationError, match="Input should be", check=_shows_all_members):
        boox.base_url = "http://random.url"  # pyright: ignore[reportAttributeAccessIssue]


def test_boox_is_not_closed_after_init(mocked_client: mock.Mock):
    boox = Boox(client=mocked_client)
    assert boox.is_closed is False


def test_boox_is_closed_after_context_exit(mocked_client: mock.Mock):
    with Boox(client=mocked_client) as boox:
        pass
    assert boox.is_closed


def test_boox_client_exposed_in_context(mocked_client: mock.Mock):
    with Boox(client=mocked_client) as boox:
        assert boox.client is mocked_client


def test_warning_when_neither_with_nor_close_used():
    with warnings.catch_warnings(record=True) as w:
        warnings.simplefilter("always")

        def create_boox():
            return Boox()

        boox = create_boox()
        del boox
        gc.collect()

    assert any(str(w.message).startswith("Boox client was not closed explicitly") for w in w)


def test_no_warning_with_context_manager():
    with warnings.catch_warnings(record=True) as w:
        warnings.simplefilter("always")

        with Boox():
            pass

    assert not any(str(w.message).startswith("Boox client was not closed explicitly") for w in w)


def test_no_warning_with_manual_close():
    with warnings.catch_warnings(record=True) as w:
        warnings.simplefilter("always")

        boox = Boox()
        boox.close()
        del boox
        gc.collect()

    assert not any(str(w.message).startswith("Boox client was not closed explicitly") for w in w)


def test_boox_close_calls_internal_method(mocker: MockerFixture, mocked_client: mock.Mock):
    boox = Boox(client=mocked_client)
    mock_close = mocker.patch.object(boox, "close")

    boox.close()
    mock_close.assert_called_once()


def test_boox_raises_on_closed_client(mocked_client: mock.Mock):
    mocked_client.is_closed = True

    with pytest.raises(ValueError, match="Cannot initialize Boox with a closed connection"):
        Boox(client=mocked_client)


def test_close_calls_client_and_sets_flag(mocker: MockerFixture, mocked_client: mock.Mock):
    boox = Boox(client=mocked_client)
    spy = mocker.spy(mocked_client, "close")

    boox.close()
    spy.assert_called_once()
    assert boox._is_closed


def test_close_skips_if_already_closed(mocker: MockerFixture, mocked_client: mock.Mock):
    boox = Boox(client=mocked_client)
    spy = mocker.spy(mocked_client, "close")
    boox._is_closed = True

    boox.close()
    spy.assert_not_called()


def test_close_skips_if_client_missing(mocker: MockerFixture, mocked_client: mock.Mock):
    boox = Boox(client=mocked_client)
    spy = mocker.spy(mocked_client, "close")
    del boox.client

    boox.close()
    spy.assert_not_called()
    assert not boox._is_closed


def test_boox_client_is_assigned_properly(mocked_client: mock.Mock):
    boox = Boox(client=mocked_client)
    assert boox.client is mocked_client


def test_boox_users_api_is_initialized(mocked_client: mock.Mock):
    boox = Boox(client=mocked_client)
    assert isinstance(boox.users, UsersApi)


def test_boox_config_users_api_is_initiated(mocked_client: mock.Mock):
    boox = Boox(client=mocked_client)
    assert isinstance(boox.config_users, ConfigUsersApi)


def test_boox_sets_default_headers(mocked_client: mock.Mock):
    boox = Boox(client=mocked_client)
    assert boox.client.headers == {"Content-Type": "application/json"}


def test_boox_token_is_unset_by_default():
    boox = Boox()
    assert not boox.token
    assert not boox.client.headers.get("Authorization")


def test_boox_token_is_extracted_from_client_headers(mocked_client: mock.Mock):
    token = uuid1()
    mocked_client.headers.update({"Authorization": f"Bearer {token!s}"})
    boox = Boox(client=mocked_client)
    assert boox.token == str(token)
    assert boox.client.headers.get("Authorization") == f"Bearer {token!s}"


def test_client_token_takes_precedence_over_inline_token(mocked_client: mock.Mock):
    client_token = uuid1()
    inline_token = uuid1()
    mocked_client.headers.update({"Authorization": f"Bearer {client_token!s}"})
    boox = Boox(client=mocked_client, token=str(inline_token))
    assert boox.token == str(client_token)
    assert boox.client.headers.get("Authorization") == f"Bearer {client_token!s}"


@pytest.mark.parametrize("wrapper", [str, _cast_to_secret_str])
def test_boox_constructor_accepts_str_and_secretstr_in_constructor(wrapper: Callable[[UUID], str | SecretStr]):
    token = uuid1()
    boox = Boox(token=wrapper(token))
    assert boox.token == str(token)
    assert boox.client.headers.get("Authorization") == f"Bearer {token!s}"


@pytest.mark.parametrize("wrapper", [str, _cast_to_secret_str])
def test_token_setter_accepts_str_and_secretstr(wrapper: Callable[[UUID], str | SecretStr]):
    token = uuid1()
    boox = Boox()
    boox.token = wrapper(token)
    assert boox.token == str(token)
    assert boox.client.headers.get("Authorization") == f"Bearer {token!s}"



import json
from http import HTTPStatus
from http.client import HTTPMessage
from json import JSONDecodeError
from urllib.error import HTTPError

import pytest
from pytest_mock import MockerFixture

from boox.client import BaseHttpClient, BaseHTTPError, BaseHttpResponse
from tests.conftest import ExpectedResponseData

# pyright: reportPrivateUsage=false

SUCCESSES = (HTTPStatus.OK, HTTPStatus.CREATED, HTTPStatus.NO_CONTENT)
FAILURES = (HTTPStatus.BAD_REQUEST, HTTPStatus.NOT_FOUND, HTTPStatus.INTERNAL_SERVER_ERROR)


def assert_response(actual: BaseHttpResponse, expected_data: ExpectedResponseData):
    assert isinstance(actual, BaseHttpResponse)
    assert actual._code == expected_data.code
    assert actual._url == expected_data.url
    assert actual._headers == expected_data.headers
    assert actual._content == expected_data.content


@pytest.mark.parametrize("code", FAILURES)
def test_http_error_str_includes_code_and_phrase(code: HTTPStatus):
    error = BaseHTTPError.__new__(BaseHTTPError)
    error.code = code
    assert str(error) == f"HTTP Error {code.value}: {code.phrase}"


@pytest.mark.parametrize("code", FAILURES)
def test_http_error_repr_includes_code_and_phrase(code: HTTPStatus):
    error = BaseHTTPError.__new__(BaseHTTPError)
    error.code = code
    assert repr(error) == f"<HTTP Error {code.value}: {code.phrase!r}>"


def test_http_error_content_decodes_empty_bytes():
    error = BaseHTTPError.__new__(BaseHTTPError)
    error._content = b''
    assert not error.content


def test_http_error_url_is_settable():
    error = BaseHTTPError.__new__(BaseHTTPError)
    error.url = (url := "https://foo.bar")
    assert error.url == url


def test_http_error_headers_are_settable():
    error = BaseHTTPError.__new__(BaseHTTPError)
    error.headers = {"foo": "bar"}
    assert error.headers == {"foo": "bar"}


def test_base_http_client_has_empty_headers():
    client = BaseHttpClient()
    assert client.headers == {}


def test_base_http_client_can_be_closed():
    client = BaseHttpClient()
    client.close()


def test_build_request_with_json_payload():
    client = BaseHttpClient()
    client.headers = {"X": "Y"}
    request = client._build_request(method="POST", url="https://foo.com", json={"foo": "bar"})
    assert request.get_full_url() == "https://foo.com"
    assert request.data == b'{"foo": "bar"}'
    assert request.get_header("X") == "Y"
    assert request.get_method() == "POST"


def test_request_error_is_caught_and_can_be_raised_for_status(mocker: MockerFixture):
    client = BaseHttpClient()
    headers = HTTPMessage()
    headers.add_header("X", "Y")
    http_error = HTTPError(code=500, url="https://foo.com", msg="Internal Server Error", hdrs=headers, fp=None)

    mocker.patch("boox.client.urlopen").side_effect = http_error
    response = client._send(request=mocker.Mock())
    with pytest.raises(BaseHTTPError) as e:
        response.raise_for_status()

    assert e.value.url == "https://foo.com"
    assert e.value.code is HTTPStatus.INTERNAL_SERVER_ERROR
    assert e.value.headers == {"X": "Y"}


def test_send_populates_response_fields_correctly(mocker: MockerFixture, mocked_urlopen: ExpectedResponseData):
    client = BaseHttpClient()
    response = client._send(request=mocker.Mock())
    assert_response(response, mocked_urlopen)


def test_get_populates_response_fields_correctly(mocker: MockerFixture, mocked_urlopen: ExpectedResponseData):
    client = BaseHttpClient()
    spy = mocker.spy(client, "_build_request")
    response = client.get(mocked_urlopen.url)

    spy.assert_called_once_with("GET", mocked_urlopen.url, None)
    assert_response(response, mocked_urlopen)


def test_post_populates_response_fields_correctly(mocker: MockerFixture, mocked_urlopen: ExpectedResponseData):
    client = BaseHttpClient()
    spy = mocker.spy(client, "_build_request")
    payload = json.loads(mocked_urlopen.content)
    response = client.post(mocked_urlopen.url, payload)

    spy.assert_called_once_with("POST", mocked_urlopen.url, payload)
    assert_response(response, mocked_urlopen)


def test_delete_populates_response_fields_correctly(mocker: MockerFixture, mocked_urlopen: ExpectedResponseData):
    client = BaseHttpClient()
    spy = mocker.spy(client, "_build_request")
    response = client.delete(mocked_urlopen.url)

    spy.assert_called_once_with("DELETE", mocked_urlopen.url, None)
    assert_response(response, mocked_urlopen)


@pytest.mark.parametrize("code", SUCCESSES)
def test_base_http_response_repr_returns_status_code(code: HTTPStatus):
    response = BaseHttpResponse.__new__(BaseHttpResponse)
    response._code = code

    assert repr(response) == f"<Response [{code.value}]>"


@pytest.mark.parametrize("code", SUCCESSES)
def test_raise_for_status_returns_response_on_success(code: HTTPStatus):
    response = BaseHttpResponse.__new__(BaseHttpResponse)
    response._code = code

    assert response.raise_for_status() is response


@pytest.mark.parametrize("code", FAILURES)
def test_raise_for_status_raises_on_failure(code: HTTPStatus):
    response = BaseHttpResponse(code=code, url="foo.com", headers={}, content=b"")

    with pytest.raises(BaseHTTPError):
        response.raise_for_status()


def test_response_json_parses_empty_object():
    response = BaseHttpResponse.__new__(BaseHttpResponse)
    response._content = b'{}'

    assert response.json() == {}


def test_response_json_raises_on_invalid_json():
    response = BaseHttpResponse.__new__(BaseHttpResponse)
    response._content = b''

    with pytest.raises(JSONDecodeError):
        response.json()



from unittest import mock
from uuid import uuid1

import pytest
from pytest_mock import MockerFixture

from boox.api.core import TokenMissingError
from boox.core import Boox
from boox.models.config_users import SyncTokenResponse
from boox.models.enums import BooxUrl
from tests.api.utils import E2EConfig
from tests.conftest import e2e


@pytest.mark.parametrize("url", list(BooxUrl))
def test_sync_token_raises_token_missing_error(mocked_client: mock.Mock, url: BooxUrl):
    with (
        Boox(client=mocked_client, base_url=url) as boox,
        pytest.raises(TokenMissingError, match="Bearer token is required to call this method"),
    ):
        boox.config_users.synchronize_token()


@pytest.mark.parametrize("url", list(BooxUrl))
def test_config_users_api_sync_token_integration(mocker: MockerFixture, mocked_client: mock.Mock, url: BooxUrl):
    token = str(uuid1())
    mocked_response = mocker.Mock()
    mocked_response.json = mocker.Mock(
        return_value={"data": None, "message": "SUCCESS", "result_code": 0, "tokenExpiredAt": 1}
    )
    mocked_response.raise_for_status.return_value = mocked_response
    mocked_client.get.return_value = mocked_response

    with Boox(client=mocked_client, base_url=url, token=token) as boox:
        result = boox.config_users.synchronize_token()

    expected_url = url.value + "/api/1/configUsers/one"
    mocked_client.get.assert_called_once_with(expected_url)
    mocked_response.json.assert_called_once()
    assert isinstance(result, SyncTokenResponse)
    assert not result.data


@e2e
def test_synchronize_token_e2e(config: E2EConfig):
    if not config.token:
        pytest.skip("Token was either not obtained or not set")

    with Boox(base_url=config.domain, token=config.token) as boox:
        response = boox.config_users.synchronize_token()

    assert not response.data
    assert response.token_expired_at



"""Example on how to fetch the session token.

It will probably be your second step.

In this example it is assumed that the verification code was obtained.
"""

from pydantic import SecretStr

from boox import Boox, BooxUrl
from boox.models.users import FetchTokenRequest, FetchTokenResponse

with Boox(base_url=BooxUrl.PUSH) as boox:
    payload = FetchTokenRequest.model_validate({"mobi": "foo@bar.com", "code": "123456"})
    response: FetchTokenResponse = boox.users.fetch_session_token(payload=payload)

# By default, received token is hidden from __str__ and __repr__.
token: SecretStr = response.data.token
# But you can reveal it using `.get_secret_value()`
print(token.get_secret_value())
